@@ -1,7 +1,8 @@
 # -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
 import sys, time, math, importlib, os, platform, os.path, configparser
 from utils import cnf
-from PyQt5.QtCore import pyqtSignal, QObject
+from PyQt5.QtCore import pyqtSignal, QObject, pyqtSlot
+from PyQt5.QtGui import QIcon
 from QtVersion import *
 showVersions()
 
@@ -14,7 +15,38 @@ if 'Windows' in pf:
 	import diodeIV, editor, filterCircuit, induction, MPU6050, npnCEout, pendulumVelocity
 	import plotIV, pnpCEout, pt100, RCtransient, RLCsteadystate, RLCtransient
 	import RLtransient, rodPendulum, scope, soundBeats, soundFreqResp, soundVelocity
-	import sr04dist, utils, logger, XYplot, i2cLogger, tof
+	import sr04dist, utils, logger, XYplot, i2cLogger, tof, advanced_logger
+
+"""
+Translations in advance for the menus:
+the top menu titles
+"""
+
+QT_TRANSLATE_NOOP('MainWindow','Device')
+QT_TRANSLATE_NOOP('MainWindow','School Expts')
+QT_TRANSLATE_NOOP('MainWindow','Electronics')
+QT_TRANSLATE_NOOP('MainWindow','Electrical')
+QT_TRANSLATE_NOOP('MainWindow','Sound')
+QT_TRANSLATE_NOOP('MainWindow','Mechanics')
+QT_TRANSLATE_NOOP('MainWindow','Other Expts')
+QT_TRANSLATE_NOOP('MainWindow','I2C Modules')
+QT_TRANSLATE_NOOP('MainWindow','PythonCode')
+
+"""
+The first submenu
+"""
+QT_TRANSLATE_NOOP('MainWindow','Reconnect')
+QT_TRANSLATE_NOOP('MainWindow','LightBackGround next time')
+QT_TRANSLATE_NOOP('MainWindow','DarkBackGround next time')
+QT_TRANSLATE_NOOP('MainWindow','Choose Language')
+
+"""
+Warning dialogs
+"""
+QT_TRANSLATE_NOOP('MainWindow','No immediate application')
+QT_TRANSLATE_NOOP('MainWindow',"Please restart the application to lighten the screen's background")
+QT_TRANSLATE_NOOP('MainWindow',"Please restart the application to darken the screen's background.")
+
 
 schoolExpts = [ 
 [QT_TRANSLATE_NOOP('MainWindow',"Voltage measurement"), ('2.1','measure-dc')],
@@ -50,7 +82,7 @@ testEquipment = [
 electronicsExpts = [ 
 [QT_TRANSLATE_NOOP('MainWindow','Diode Characteristics'),('3.11','diodeIV')],
 [QT_TRANSLATE_NOOP('MainWindow','NPN Output Characteristics'),('3.12','npnCEout')],
-[QT_TRANSLATE_NOOP('MainWindow','PNP Output Characteristics'),'pnpCEout'],
+[QT_TRANSLATE_NOOP('MainWindow','PNP Output Characteristics'),('3.13','pnpCEout')],
 #[QT_TRANSLATE_NOOP('MainWindow','AM and FM'), 'amfm']
 ]
 
@@ -61,7 +93,7 @@ electronicsExptsScope = [
 [QT_TRANSLATE_NOOP('MainWindow','Diode Clipping'),('3.3','clipping')],
 [QT_TRANSLATE_NOOP('MainWindow','Diode Clamping'),('3.4','clamping')],
 [QT_TRANSLATE_NOOP('MainWindow','IC555 Multivibrator'),('3.5','osc555')],
-[QT_TRANSLATE_NOOP('MainWindow','Transistor Amplifier (CE)'),'npnCEamp'],
+[QT_TRANSLATE_NOOP('MainWindow','Transistor Amplifier (CE)'),('3.14','npnCEamp')],
 [QT_TRANSLATE_NOOP('MainWindow','Inverting Amplifier'),('3.6','opamp-inv')],
 [QT_TRANSLATE_NOOP('MainWindow','Non-Inverting Amplifier'),('3.7','opamp-noninv')],
 [QT_TRANSLATE_NOOP('MainWindow','Integrator using Op-Amp'),('3.8','opamp-int')],
@@ -70,13 +102,13 @@ electronicsExptsScope = [
 ]
 
 electricalExpts = [ 
-[QT_TRANSLATE_NOOP('MainWindow','Plot I-V Curve'),'plotIV'],
-[QT_TRANSLATE_NOOP('MainWindow','XY Plotting'),'XYplot'],
+[QT_TRANSLATE_NOOP('MainWindow','Plot I-V Curve'),('4.01','plotIV')],
+[QT_TRANSLATE_NOOP('MainWindow','XY Plotting'),('4.02','XYplot')],
 [QT_TRANSLATE_NOOP('MainWindow','RLC Steady state response'),('4.1','RLCsteadystate')],
 [QT_TRANSLATE_NOOP('MainWindow','RC Transient response'),('4.2','RCtransient')],
 [QT_TRANSLATE_NOOP('MainWindow','RL Transient response'),('4.3','RLtransient')],
-[QT_TRANSLATE_NOOP('MainWindow','RLC transient response'),('4.4','RLCtransient')],
-[QT_TRANSLATE_NOOP('MainWindow','Frequency Response of Filter Circuit'),('4.5','filterCircuit')],
+[QT_TRANSLATE_NOOP('MainWindow','RLC transient response'),('4.5','RLCtransient')],
+[QT_TRANSLATE_NOOP('MainWindow','Frequency Response of Filter Circuit'),('4.6','filterCircuit')],
 [QT_TRANSLATE_NOOP('MainWindow','Electromagnetic Induction'),('4.7','induction')]
 ]
 
@@ -91,19 +123,20 @@ mechanicsExpts = [
 [QT_TRANSLATE_NOOP('MainWindow','Pendulum Wavefrorm'),('6.2','pendulumVelocity')],
 [QT_TRANSLATE_NOOP('MainWindow','Driven Pendulum resonance'),('6.3','driven-pendulum')],
 [QT_TRANSLATE_NOOP('MainWindow','Distance by HY-SRF04 Echo module'), ('6.4','sr04dist')],
-[QT_TRANSLATE_NOOP('MainWindow','Gravity by Time of Flight'), 'tof']
+[QT_TRANSLATE_NOOP('MainWindow','Gravity by Time of Flight'), ('6.10','tof')]
 ]
 
 otherExpts = [ 
 [QT_TRANSLATE_NOOP('MainWindow','Temperatue, PT100 Sensor'), ('6.5','pt100')],
-[QT_TRANSLATE_NOOP('MainWindow','Data Logger'), 'logger']
+[QT_TRANSLATE_NOOP('MainWindow','Data Logger'), ('6.6','logger')],
+[QT_TRANSLATE_NOOP('MainWindow','Advanced Data Logger'), ('6.7','advanced_logger')]
 ]
 
 modulesI2C = [ 
-[QT_TRANSLATE_NOOP('MainWindow','Magnetic Hysterisis (MPU925x Sensor)'),'BHCurve'],
-[QT_TRANSLATE_NOOP('MainWindow','Luminosity(TSL2561) Logger'),'lightsensorlogger'],
-[QT_TRANSLATE_NOOP('MainWindow','MPU-6050 Acccn, Velocity and Temp'), 'MPU6050'],
-[QT_TRANSLATE_NOOP('MainWindow','General Purpose I2C Sensors'), 'i2cLogger']
+[QT_TRANSLATE_NOOP('MainWindow','Magnetic Hysterisis (MPU925x Sensor)'),('6.90', 'BHCurve')],
+[QT_TRANSLATE_NOOP('MainWindow','Luminosity(TSL2561) Logger'),('6.91', 'lightsensorlogger')],
+[QT_TRANSLATE_NOOP('MainWindow','MPU-6050 Acccn, Velocity and Temp'), ('6.92', 'MPU6050')],
+[QT_TRANSLATE_NOOP('MainWindow','General Purpose I2C Sensors'), ('6.93', 'i2cLogger')]
 ]
 
 pythonCodes = [ 
@@ -202,28 +235,45 @@ class MainWindow(QMainWindow):
 	hwin = None
 	
 	uncheckHelpBox = pyqtSignal()
+	setEditorText = pyqtSignal(str)
+	setConfigText = pyqtSignal(str)
 	
 	def closeEvent(self, e):
 		if self.hwin != None:
 			self.hwin.close()
 
-	def __init__(self, lang):
+	def __init__(self, lang, app, tr_eyes, tr_qt):
 		"""
 		The constructor.
 		:param lang: the autodetected language, which comes from shell variables
+		:type  lang: str
+		:param app: pointer to the Application
+		:type  app: QApplication
+		:param tr_eyes: translator to localize eyes17
+		:type  tr_eyes: QTranslator
+		:param tr_qt: translator to localize Qt5
+		:type  tr_qt: QTranslator
 		"""
 		QMainWindow.__init__(self)
+		self.lang=None # this will be set later, after self.translate() tries
+		self.app=app
+		self.tr_eyes=tr_eyes
+		self.tr_qt=tr_qt
 
-		config = configparser.ConfigParser()
-		config.read(cnf)
+		self.conf = configparser.ConfigParser()
+		self.conf.read(cnf)
+		
 		try:
-			self.translators = self.translate(config['ScreenTheme']['language'])
-			self.lang = config['ScreenTheme']['language']
+			self.translate(self.conf['ScreenTheme']['language'])
+			self.lang = self.conf['ScreenTheme']['language']
 		except:
-			self.translators = self.translate(lang)
+			self.translate(lang)
 			self.lang=lang
 			
+		self.init_UI()
 		self.uncheckHelpBox.connect(self.uncheckTheHelpBox)
+		self.setEditorText.connect(self.updateEditor)
+		self.setConfigText.connect(self.updateConfig)
 
 	def uncheckTheHelpBox(self):
 		"""
@@ -239,7 +289,7 @@ class MainWindow(QMainWindow):
 		self._x = 100
 		self._y = 10
 		palette = QPalette()								# background color
-		palette.setColor(QPalette.Background, QColor(61,168,165)) #("#99ccff")) "#88bbcc"
+		palette.setColor(QPalette.Background, QColor(81,188,185)) #("#99ccff")) "#88bbcc"
 		self.setPalette(palette)	
 
 		self.helpCB = QCheckBox(self.tr('Enable PopUp Help Window'))
@@ -255,12 +305,47 @@ class MainWindow(QMainWindow):
 		self.show()
 		self.move(20, 20)
 		
-		
+	def updateEditor(self,text):
+		if self.expName[1] == 'editor':
+			text = text.replace('import eyes17.eyes','#import eyes17.eyes')
+			text = text.replace('p = eyes17.eyes.open()','#p = eyes17.eyes.open() #Uncomment when running script in standalone mode')
+			self.expWidget.Edit.setText(text)
+			self.activateWindow()
+
+	def updateConfig(self,text):
+		if self.expName[1] == 'advanced_logger':
+			self.expWidget.setConfig(text)
+			self.activateWindow()
+
+	class editorHandler(QObject):
+		def __init__(self,sigEditor,sigConfig):
+			QMainWindow.__init__(self)
+			self.sigEditor = sigEditor
+			self.sigConfig = sigConfig
+
+		@pyqtSlot(str)
+		def update(self,value):
+			self.sigEditor.emit(value)
+
+		@pyqtSlot(str)
+		def config(self,value):
+			self.sigConfig.emit(value)
 
 	def showHelp(self):
 		if self.helpCB.isChecked() == True:
 			if self.hwin == None:
 				self.hwin = helpWin(self, (self.title,self.hlpName), self.lang)
+				if(self.hlpName[1] in ['editor','advanced_logger']):
+					try:
+						from PyQt5.QtWebChannel import QWebChannel
+						self.channel = QWebChannel()
+						self.handler = self.editorHandler(self.setEditorText,self.setConfigText)
+						self.channel.registerObject('handler', self.handler)
+						self.hwin.page().setWebChannel(self.channel)
+						print('online help available')
+					except Exception as e:
+						print(e)
+
 			self.hwin.show()
 		else:
 			if self.hwin != None: self.hwin.hide()
@@ -298,9 +383,9 @@ class MainWindow(QMainWindow):
 		explib = importlib.import_module(module_name)
 		try:
 			if self.expWidget != None:
-				self.expWidget.timer.stop()     # Stop the timer loop of current widget			
+				self.expWidget.timer.stop()	 # Stop the timer loop of current widget			
 			self.hwin = None
-			self.expWidget= None 			    # Let python delete it
+			self.expWidget= None				 # Let python delete it
 			w = explib.Expt(p) 
 			self.setWindowTitle(self.tr(e[0]))
 			self.setCentralWidget(w)
@@ -316,8 +401,9 @@ class MainWindow(QMainWindow):
 		return
 		
 	def runCode(self, e):
-		if self.expName != 'editor':
-			self.callExpt( ('Python Coding', 'editor'))
+		if self.expName != 'editor': #Moved here from some other non coding expt
+			self.hlpName = e
+			self.callExpt( ['Python Coding', ('9.0','editor')])
 		self.expWidget.mycode = e[1]
 		self.expWidget.update()
 
@@ -329,10 +415,10 @@ class MainWindow(QMainWindow):
 		@param key for example: 'Background'
 		@param value the text to assign to the key, for example: 'dark'
 		"""
-		config = configparser.ConfigParser()
-		config.read(cnf)
-		config[section][key] = value
-		with open(cnf,"w") as out: config.write(out)
+		self.conf = configparser.ConfigParser()
+		self.conf.read(cnf)
+		self.conf[section][key] = value
+		with open(cnf,"w") as out: self.conf.write(out)
 		return
 	
 	def setWBG(self):
@@ -360,6 +446,8 @@ class MainWindow(QMainWindow):
 		return
 	
 	def makeMenu(self):
+		imagePath = os.path.join(os.path.dirname(
+			os.path.abspath(__file__)),'images')
 		bar = self.menuBar()
 		bar.clear() # reset all menu actions
 		mb = bar.addMenu(self.tr("Device"))
@@ -367,9 +455,13 @@ class MainWindow(QMainWindow):
 		mb.addAction(self.tr('LightBackGround next time'), self.setWBG)
 		mb.addAction(self.tr('DarkBackGround next time'), self.setBBG)
 		sm = mb.addMenu(self.tr("Choose Language"))
+		sm.setIcon(QIcon(os.path.join(imagePath, "UN_emblem_blue.svg")))
 		for e in languages:
-			sm.addAction(e,  lambda item=e: self.setLanguage(item))	
-
+			action = sm.addAction(e,  lambda item=e: self.setLanguage(item))
+			flag=os.path.join(imagePath, f"{e}.svg")
+			if os.path.exists(flag):
+				action.setIcon(QIcon(flag))
+				action.setIconVisibleInMenu(True)
 		em = bar.addMenu(self.tr("School Expts"))
 		for e in schoolExpts:
 			em.addAction(self.tr(e[0]),  lambda item=e: self.scope_help(item))	
@@ -409,66 +501,74 @@ class MainWindow(QMainWindow):
 			self.setConfig('ScreenTheme', 'language', l)
 			self.translators=self.translate(l)
 			self.lang=l
+			self.init_UI()
 			return
 
 	def reconnect(self):
-		global p
+		global p,eyes
 		try:
 			p.H.disconnect()
 		except:
 			pass
 		p=eyes.open()
+		if self.expWidget is None:
+			explib = importlib.import_module('scope')
+			self.expWidget = explib.Expt(p) 
+			self.setCentralWidget(self.expWidget)
+			self.setWindowTitle(self.tr('Oscilloscope'))
+			self.expName = 'scope'
+
 		self.expWidget.p = p
 		self.expWidget.msg('')
 		if p != None: 
+			print('recovering...',self.expName)
 			if self.expName == 'scope':
 				self.expWidget.recover()
 		
 	# translation stuff
 	def translate(self, lang = None):
-		global app,t,t1
-
 		try:
-			app.removeTranslator(t)
-			app.removeTranslator(t1)
+			self.app.removeTranslator(self.tr_eyes)
+			self.app.removeTranslator(self.tr_qt)
 		except:
 			pass
 
 		if lang is None:
 			lang=QLocale.system().name()
-		t=QTranslator()
-		t.load("lang/"+lang, os.path.dirname(__file__))
-		app.installTranslator(t)
-		t1=QTranslator()
-		t1.load("qt_"+lang,
+		self.tr_eyes=QTranslator()
+		self.tr_eyes.load("lang/"+lang, os.path.dirname(__file__))
+		self.app.installTranslator(self.tr_eyes)
+		self.tr_qt=QTranslator()
+		self.tr_qt.load("qt_"+lang,
 			QLibraryInfo.location(QLibraryInfo.TranslationsPath))
-		app.installTranslator(t1)
-		self.init_UI()
+		self.app.installTranslator(self.tr_qt)
 		self.uncheckHelpBox.emit()
-		return t,t1
 
+def run():
+	# Program starts here
+	global app,p,eyes
+	import eyes17.eyes as eyes
+	p = eyes.open()
+	if p != None: 
+		p.set_sine(1000)
+		p.set_sqr1(-1)
+		p.set_pv1(0)
+		p.set_pv2(0)
+		p.set_state(OD1=0)
 
-# Program starts here
-import eyes17.eyes as eyes
-p = eyes.open()
-if p != None: 
-	p.set_sine(1000)
-	p.set_sqr1(-1)
-	p.set_pv1(0)
-	p.set_pv2(0)
-	p.set_state(OD1=0)
+	app = QApplication(sys.argv)
+	# translation stuff
+	lang=QLocale.system().name()
+	t=QTranslator()
+	t.load("lang/"+lang, os.path.dirname(__file__))
+	app.installTranslator(t)
+	t1=QTranslator()
+	t1.load("qt_"+lang,
+		QLibraryInfo.location(QLibraryInfo.TranslationsPath))
+	app.installTranslator(t1)
 
-app = QApplication(sys.argv)
+	mw = MainWindow(lang, app, t, t1)
+	sys.exit(app.exec_())
 
-# translation stuff
-lang=QLocale.system().name()
-t=QTranslator()
-t.load("lang/"+lang, os.path.dirname(__file__))
-app.installTranslator(t)
-t1=QTranslator()
-t1.load("qt_"+lang,
-	QLibraryInfo.location(QLibraryInfo.TranslationsPath))
-app.installTranslator(t1)
-
-mw = MainWindow(lang)
-sys.exit(app.exec_())
+if __name__ == '__main__':
+	run()
